/**
   The static method parsePrefix() is a recursive descent parser for
   strings that hold the prefix notation of an arithmetic expression.

   Here is the BNF grammar for prefix notation expressions.

      Prefix -> BinOp Prefix Prefix
              | UnOp Prefix
              | Number

       BinOp -> '+' | '-' | '*' | '/'

       UnOp  -> 'neg' | 'sqrt'

      Number -> any valid Java double

   In a recursive descent parser, every production is handled by a method.
   If a production is recursive, then the method will be recursive.
   If two productions are mutually recursive, then the two methods will
   be mutually recursive. Since this grammar has four productions, the
   parser consists of four methods, getPrefix(), getBinOP(), getUnOp(),
   and getNumber().

   Input strings are "tokenized" by the very bare bones tokenizer
   contained in the file Tokenizer.java. This tokenizer requires
   that all tokens in the input string be surrounded by white space
   (which makes the tokenizer almost trivial to write).


   NOTE: This parser does not "do" anything. It does not create a
   an expression tree nor a parse tree. This parser is just a
   "recognizer" for the language. It will throw an exception if
   a string cannot be parsed.
*/

public class ParsePrefix
{
   public static void parsePrefix(String infix)
   {
      Tokenizer tokens = new Tokenizer(infix); // Tokenizer defined in Tokenizer.java

      getPrefix(tokens);        // parse the token stream

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new Error("syntax error: unexpected input: "+tokens.peekToken()+"\n"+tokens);
   }//parsePrefix2ParseTree()



   private static void getPrefix(Tokenizer tokens)
   {
      String token = tokens.peekToken();  // one token look ahead

      if ( "+-*/^".indexOf(token) != -1 ) // binary operator case
      {
         getBinOp(tokens);  // parse the operator
         getPrefix(tokens); // parse the first operand
         getPrefix(tokens); // parse the second operand
      }
      else if ( "neg sqrt".indexOf(token) != -1 ) // unary operator case
      {
         getUnOp(tokens);   // parse the operator
         getPrefix(tokens); // parse the (only) operand
      }
      else // operand case (this case terminates the recursion)
      {
         getNumber(tokens); // parse the number
      }
    }//getPrefix()


   private static void getBinOp(Tokenizer tokens)
   {
      String token = tokens.nextToken(); // consume one token
   }//getBinOp()


   private static void getUnOp(Tokenizer tokens)
   {
      String token = tokens.nextToken(); // consume one token
   }//getUnOp()


   private static void getNumber(Tokenizer tokens)
   {
      String token = tokens.nextToken(); // consume one token
   }//getNumber()

}//ParsePrefix
