/**
   The static method parsePrefix() is a recursive descent parser for
   strings that hold the prefix notation of an arithmetic expression.

   Here is the BNF grammar for prefix notation expressions.

      Prefix -> BinOp Prefix Prefix
              | UnOp Prefix
              | Number

       BinOp -> '+' | '-' | '*' | '/'

       UnOp  -> 'neg' | 'sqrt'

      Number -> any valid Java double

   Since this grammar has only one recursive production, it is possible to
   write the whole parser as a single recursive method, getPrefix().

   This parser creates an expression tree (that is, an "abstract syntax tree")
   for the prefix expression.
*/

public class ParsePrefix2ASTSingleMethod
{
   public static BTree parsePrefix(String infix)
   {
      Tokenizer tokens = new Tokenizer(infix); // Tokenizer defined in Tokenizer.java

      BTree result = getPrefix(tokens);        // parse the token stream

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new Error("syntax error: unexpected input: "+tokens.peekToken()+"\n"+tokens);

      return result;
   }//parsePrefix()


   private static BTree getPrefix(Tokenizer tokens)
   {
      BTree result = null;

      String token = tokens.nextToken(); // consume one token

      if ( "+-*/^".indexOf(token) != -1 ) // binary operator case
      {
         result = new BTree( token, getPrefix(tokens), getPrefix(tokens) );
      }
      else if ( "neg sqrt".indexOf(token) != -1 ) // unary operator case
      {
         result = new BTree( token, null, getPrefix(tokens) );
      }
      else // operand case (this case terminates the recursion)
      {
         result = new BTree( token );
      }
      return result;
   }//getPrefix()

}//ParsePrefix2ASTSingleMethod
