/*
   Here is an unambiguous, "right associative", BNF grammar
   (and also an EBNF grammar) for the language

      L = { a, a+a, a+a+a, a+a+a+a, ... }.


   BNF:    expr -> a + expr | a   (notice that this is right recursive)


   EBNF:   expr -> a [ + expr ]   (notice that this is still right recursive)


   Using this BNF grammar you parse the string "a+a+a+a" as if it were
   "grouped" as a+(a+(a+a))

   This is a "recursive descent" parser based on the EBNF grammar. Since there
   is only one production in the grammar, the parser is a single method, getExpr().

   Notice how the "or" symbol in the BNF (and the "option" symbol in the EBNF)
   becomes an if-statement in the parser. If the token stream has another token,
   it is assumed to be a '+' operator and it is read from the stream, and then
   another expression is consumed recursively.

 : This parser builds the expression tree from the top down. Notice how the
   constructor call in line 47 builds the final expression tree by making a
   recursive call to getExpr() and using the returned tree as the right
   sub-tree of the final expression tree.
*/

public class ParserL1
{
   public static BTree parseL(String stringInL)
   {
      Tokenizer tokens = new Tokenizer(stringInL);  // Tokenizer is defined in Tokenizer.java

      return getExpr(tokens);
   }//parseL()


   public static BTree getExpr(Tokenizer tokens)
   {
      BTree result = new BTree( tokens.nextToken() ); // consume at least one 'a'
      if ( tokens.hasToken() )               // conditionally look for more terms
      {
         String token = tokens.nextToken();  // consume '+'
         result = new BTree(token, result, getExpr(tokens)); // recursively consume another expr
      }
      return result;
   }//getExpr()



   public static void main(String[] args)
   {
      String[] stringsInL = { "a", "a + a", "a + a + a", "a + a + a + a", "a + b + c + d" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         BTree btree = parseL(stringsInL[i]);
         System.out.println( "//   " + stringsInL[i] + "  ==>  " + btree );
         System.out.println( BTree2dot.btree2dot(btree) + "\n" );
      }
   }//main()

}//ParserL1


/*


*/