/*
   Here is an unambiguous, "left associative", BNF grammar
   (and also three EBNF grammars) for the language

      L = { a, a+a, a+a+a, a+a+a+a, ... }.


   BNF:    expr -> expr + a | a  (notice that this is left recursive)


   EBNF:   expr -> [ expr + ] a   (notice that this is still left recursive)


   EBNF:   expr -> ( a + )* a     (we removed (or "eliminated") the recursion)


   Using this BNF grammar you parse the string "a+a+a+a" as if it were
   "grouped" as ((a+a)+a)+a

   We cannot write a recusive descent parser using these grammars because
   the left associativety would cause infinite recursion (the getExpr()
   method would immediately, and repeatedly, call itself). But we can use
   the following grammar to write a parser.

   EBNF:   expr -> a ( + a )*

   This grammar was derived from the right recursive BNF grammar for L.
   Suprisingly, we use this grammar to write a "recursive descent" grammar
   that is left associative. (But notice that the parser below is not
   recursive, because this grammar is itself not recursive.) It is important
   to realize that the above EBNF grammar can be the grammar for either a
   right or left associative parser.
*/

public class ParserL2
{
   public static BTree parseL(String stringInL)
   {
      Tokenizer tokens = new Tokenizer(stringInL);  // Tokenizer is defined in Tokenizer.java

      return getExpr(tokens);
   }//parseL()


   public static BTree getExpr(Tokenizer tokens)
   {
      BTree result = new BTree( tokens.nextToken() ); // consume at least one 'a'
      while( tokens.hasToken() )              // iteratively consume more terms
      {
         String token1 = tokens.nextToken();  // consume a '+'
         BTree token2 = new BTree( tokens.nextToken() );  // consume a 'a'
         result = new BTree(token1, result, token2);  // old result on the left
      }
      return result;
   }//getExpr()



   public static void main(String[] args)
   {
      String[] stringsInL = { "a", "a + a", "a + a + a", "a + a + a + a", "a + b + c + d" };

      for (int i = 0; i < stringsInL.length; i++)
      {
         BTree btree = parseL(stringsInL[i]);
         System.out.println( "//   " + stringsInL[i] + "  ==>  " + btree );
         System.out.println( BTree2dot.btree2dot(btree) + "\n" );
      }
   }//main()

}//ParserL2


/*
   NOTE1: This parser builds the parse tree from the bottom up.
   Notice how in line 52, the constructor call
         result = new BTree(token1, result, token2);
   builds the new (larger) parse tree by making the previous
   parse tree the left sub-tree.


   NOTE2: If we change the constructor call mentioned above so
   that it uses the previous parse tree as the right sub-tree
   of the new parse tree, like this
         result = new BTree(token1, token2, result);
   AND if we make the tokenizer read the input string from
   right to left, then this algorithm does a right associative
   parse of the expression. This shows that the Kleene star
   grammar that this algorithm was derived from
           expr -> a ( + a )*
   does not by itself define any associativity for the operator.
   The associativity doesn't come from the grammar in this case.
   it comes from how we choose to implement the grammar.
   (It's easy to make the "tokenizer" Tokenizer.java read
   a string from right to left. Try it.)
*/